#define _SPARSEC_C_
#define _ADOLC_SRC_
#include <stdlib.h>
/*
   --------------------------------------------------------------
   File sparsec.c of ADOL-C version 1.8.0         as of Nov/30/98
   --------------------------------------------------------------
   All "Easy To Use" C interfaces of SPARSE package

   Last changes:
        981130 olvo: newly created from driversc.c

   --------------------------------------------------------------
*/


/****************************************************************************/
/*                                                                 INCLUDES */
#include "dvlparms.h" /* Developers Parameters */
#include "usrparms.h" /* Users Parameters      */
#include "sparse.h"
#include "jacutils.h"

#include <stdio.h>
#include <math.h>

#ifdef __cplusplus
#include <malloc.h>
extern "C" {
#endif


/*---------------------------------------------------------------------------*/
/*                                                         jacobian pattern  */
/*                                                                           */

int jac_pat( 
      short  tag,       /* tape identification                               */
      int    depen,     /* number of dependent variables                     */
      int    indep,     /* number of independent variables                   */
      double *basepoint,/* independant variable values                       */
      unsigned short **rb,    
      /* (*rb)[0] = number of blocks of dependent variables                  */
      /* dependent variable j=0..depen-1 belongs to row block (*rb)[1+j],    */
      /* if (*rb) == NULL  each dependent variable will be considered 
           as a block of dependent variables                                 */
      /* rb is returned in order to keep compatible to further developments  */
      unsigned short **cb,
      /* (*cb)[0] = number of blocks of independent variables                */
      /* independent variable i=0..indep-1 belongs to column block (*cb)[1+i]*/
      /* if (*cb) == NULL  each independent variable will be considered 
           as a block of independent variables                               */
      /* cb is returned in order to keep compatible to further developments  */
      unsigned short ***crs,     
      /* returned compressed row block-index storage 
           *[(*rb)[0]][non-zero blocks of independent variables per row]*/
      /* (*crs)[depen. bl.][0] = non-zero indep. bl. w.r.t current depen. bl.  
         (*crs)[depen. bl.][1 .. (*crs)[depen. bl.][0]] :                 
           indeces of non-zero blocks of independent variables  with
             respect to the current block of dependent variables             */
      int    options[2] /* control options                                   */
      /*          options[0] : way of bit pattern propagation
                             0 - automatic detection (default)
                             1 - forward mode 
                             2 - reverse mode
                  options[1] : test the computational graph control flow
                             0 - safe mode (default)
                             1 - tight mode                                  */
              
      )
{
  int             rc= -1, i, j;
  unsigned short  depen_blocks, indep_blocks, *rowbl, *colbl; 
  int             ctrl_options[3];

  if (rb == NULL)
    {
      fprintf(DIAG_OUT,"ADOL-C user error in jac_pat(...) : "
                       "parameter &rb is NULL !\n");
      exit(-1);
    }
  if (cb == NULL)
    {
      fprintf(DIAG_OUT,"ADOL-C user error in jac_pat(...) : "
                       "parameter &cb is NULL !\n");
      exit(-1);
    }

  if (*rb == NULL)
    {
      *rb = myalloc1_ushort(1+depen);
      rowbl = *rb;
      *rowbl++ = depen;
      for (j=0; j<depen; j++)
        *rowbl++ = j;
    }
  if (*cb == NULL)
    {
      *cb = myalloc1_ushort(1+indep);
      colbl = *cb;
      *colbl++ = indep;
      for (i=0; i<indep; i++)
        *colbl++ = i;
    }
  
  rowbl = *rb;
  depen_blocks = *rowbl++;
  for (j=0; j<depen; j++)
    if ( *rowbl++ >= depen_blocks )
      {
        fprintf(DIAG_OUT,"ADOL-C user error in jac_pat(...) : "
                "bad dependent block index rb[%i]=%i >= rb[0]=%i !\n", 
                j+1, *(--rowbl), depen_blocks);
        exit(-1);
      }
  colbl = *cb;
  indep_blocks = *colbl++;
  for (i=0; i<indep; i++)
    if ( *colbl++ >= indep_blocks )
      {
        fprintf(DIAG_OUT,"ADOL-C user error in jac_pat(...) : "
                "bad independent block index cb[%i]=%i >= cb[0]=%i !\n", 
                i+1, *(--colbl), indep_blocks);
        exit(-1);
      }

  if ( ! ((*crs) = (unsigned short**)calloc(depen_blocks, sizeof(unsigned short*))) )
    {
      fprintf(DIAG_OUT,"ADOL-C error, "__FILE__
              ":%i : \njac_pat(...) unable to allocate %i bytes !\n",
              __LINE__, depen_blocks*sizeof(unsigned short*));
      exit(-1);
    }

  if (( options[0] < 0 ) || (options[0] > 2 ))
     options[0] = 0; /* default */
  if (( options[1] < 0 ) || (options[1] > 1 ))
     options[1] = 0; /* default */
  ctrl_options[0] = options[0];
  ctrl_options[1] = options[1];
  ctrl_options[2] = 0; /* no output intern */

  rc = block_pattern( tag, depen, indep,  basepoint,
                      *rb, *cb, *crs, ctrl_options);

  return(rc);
}

/****************************************************************************/
/*                                                               THAT'S ALL */

#ifdef __cplusplus
}
#endif

#undef _ADOLC_SRC_
#undef _SPARSEC_C_
